//------------------------------------------------------------------------------
// Digis TDSShape Component Version 1.0
//
// For the latest version and documentation vist the Digis web site at:
//   http://www.copseystrain.com/digis/
//
// Please send comments and suggestions to:
//   jcop@copseystrain.com
//
// Copyright  1999, 2000 Copsey Strain, Inc.
//------------------------------------------------------------------------------
// This component may be used freely for private or commercial uses. If any
// modifications are made to the component you must share these modifications
// with Copsey Strain, Inc. This component may not be sold in any form. The
// component can be redistributed as long as the entire original package is
// distributed and no fee is charged.
//------------------------------------------------------------------------------

unit dsshape;

interface

uses Windows, classes, controls, graphics, extctrls;

type

  TDSShapeType = (stHLine, stVLine, stHorizontalLine1,
                  stHorizontalLine2, stSquare, stEllipse,
                  stCircle, stRectangle);

  TDSShapeFill = (fsNone, fsHGradient, fsVGradient,
                  fsImageTile, fsImageCenter,
                  fsImageStretch, fsSolid);

  TDSShape = class(TGraphicControl)
  private
    FStarted: boolean;
    FLineWidth: integer;
    FLineColor: TColor;
    FShapeType: TDSShapeType;
    FFillStyle: TDSShapeFill;
    FFillColor: TColor;
    FFillColor2: TColor;
    FPaintRect: TRect;
    FMaskBitmap: TBitmap;
    FDrawBitmap: TBitmap;
    FImage: TPicture;
    procedure CreateMaskBitmap;
    procedure FreeMaskBitmap;
    procedure MaskDrawBitmap;
    procedure DrawTransparent;
    procedure GetRGB(c: integer; var r, g, b: integer);
    procedure SetMaskPenBrush;
    procedure SetPenBrush;
    procedure SetImage(value: TPicture);
    procedure SetLineWidth(value: integer);
    procedure SetLineColor(value: TColor);
    procedure SetShapeType(value: TDSShapeType);
    procedure SetFillStyle(value: TDSShapeFill);
    procedure SetFillColor(value: TColor);
    procedure SetFillColor2(value: TColor);
    procedure CalcPaintRect;
    procedure Draw_Rectangle;
    procedure Draw_Square;
    procedure Draw_Ellipse;
    procedure Draw_Circle;
    procedure Draw_VLine;
    procedure Draw_HLine;
    procedure Draw_HorizontalLine1;
    procedure Draw_HorizontalLine2;
    procedure FillDraw_Rectangle;
    procedure FillDraw_Square;
    procedure FillDraw_Ellipse;
    procedure FillDraw_Circle;
    procedure FillDrawBitmap;
    procedure Fill_HGradient;
    procedure Fill_VGradient;
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property OnMouseDown;
    property OnMouseUp;
    property OnMouseMove;
    property OnDblClick;
    property OnClick;
    property Anchors;
    property LineWidth: integer read FLineWidth write SetLineWidth default 1;
    property LineColor: TColor read FLineColor write SetLineColor default clBlack;
    property ShapeType: TDSShapeType read FShapeType write SetShapeType default stRectangle;
    property FillStyle: TDSShapeFill read FFillStyle write SetFillStyle default fsSolid;
    property FillColor: TColor read FFillColor write SetFillColor default clWhite;
    property FillColor2: TColor read FFillColor2 write SetFillColor2 default clWhite;
    property Image: TPicture read FImage write SetImage;
  end;

procedure Register;

implementation

constructor TDSShape.Create(AOwner: TComponent);
begin
  FMaskBitmap := nil;
  FImage := TPicture.create;
  FShapeType := stRectangle;
  FFillStyle := fsSolid;
  FFillColor := clWhite;
  FFillColor2 := clWhite;
  SetLineWidth(1);
  SetLineColor(clBlack);
  inherited;
  FStarted := true;
end;

destructor TDSShape.Destroy;
begin
  FImage.Free;
  FreeMaskBitmap;
  inherited;
end;

procedure TDSShape.SetImage(value: TPicture);
begin
  FImage.Assign(value);
  Refresh;
end;

procedure TDSShape.CreateMaskBitmap;
begin
  FDrawBitmap := TBitmap.Create;
  FDrawBitmap.width := width;
  FDrawBitmap.height := height;
  if FMaskBitmap = nil then begin
    FMaskBitmap := TBitmap.Create;
    FMaskBitmap.pixelformat := pf4bit;
  end;
  FMaskBitmap.width := width;
  FMaskBitmap.height := height;
  with FMaskBitmap.canvas do begin
    brush.color := clWhite;
    brush.style := bsSolid;
    pen.style := psClear;
    fillrect(rect(0, 0, width, height));
  end;
end;

procedure TDSShape.FreeMaskBitmap;
begin
  if FMaskBitmap <> nil then begin
    FMaskBitmap.Free;
    FMaskBitmap := nil;
  end;
end;

procedure TDSShape.MaskDrawBitmap;
begin
  FDrawBitmap.canvas.copymode := cmSrcPaint;
  FDrawBitmap.canvas.draw(0, 0, FMaskBitmap);
end;

procedure TDSShape.DrawTransparent;
begin
  with canvas do begin
    copymode := cmMergePaint;
    draw(0, 0, FMaskBitmap);
    copymode := cmSrcAnd;
    draw(0, 0, FDrawBitmap);
  end;
end;

procedure TDSShape.SetLineWidth(value: integer);
var
  ox, oy: integer;
begin
  if Value <> FLineWidth then begin
    FLineWidth := Value;
    Refresh;
  end;
end;

procedure TDSShape.SetLineColor(value: TColor);
begin
  if Value <> FLineColor then begin
    FLineCOlor := value;
    Refresh;
  end;
end;

procedure TDSShape.SetShapeType(value: TDSShapeType);
begin
  if Value <> FShapeType then begin
    FShapeType := value;
    Refresh;
  end;
end;

procedure TDSShape.SetFillStyle(value: TDSShapeFill);
begin
  if Value <> FFillStyle then begin
    FFillStyle := value;
    Refresh;
  end;
end;

procedure TDSShape.SetFillColor(value: TColor);
begin
  if Value <> FFillColor then begin
    FFillColor := value;
    Refresh;
  end;
end;

procedure TDSShape.SetFillColor2(value: TColor);
begin
  if Value <> FFillColor2 then begin
    FFillColor2 := value;
    Refresh;
  end;
end;

procedure TDSShape.FillDrawBitmap;
var
  x, y: integer;
begin
  case FFillStyle of
    fsHGradient: Fill_HGradient;
    fsVGradient: Fill_VGradient;
    fsImageTile, fsImageCenter, fsImageStretch:
      begin
        with FDrawBitmap.canvas do begin
          brush.color := FFillColor;
          brush.style := bsSolid;
          pen.style := psClear;
          fillrect(rect(0, 0, width, height));
          if FImage.Graphic <> nil then begin
            if FImage.Graphic.Empty = false then begin
              if FFillStyle = fsImageCenter then begin
                x := (width - FImage.Width) div 2;
                y := (height - FImage.height) div 2;
                draw(x, y, FImage.Graphic);
              end
              else if FFillStyle = fsImageStretch then begin
                stretchdraw(FPaintRect, FImage.Graphic);
              end
              else if FFillStyle = fsImageTile then begin
                y := FPaintRect.Top;
                while y < FPaintRect.Bottom do begin
                  x := FPaintRect.Left;
                  while x < FPaintRect.Right do begin
                    draw(x, y, FImage.Graphic);
                    inc(x, FImage.Width);
                  end;
                  inc(y, FImage.Height);
                end;
              end;
            end;
          end;
        end;
      end;
  end;
end;

procedure TDSShape.GetRGB(c: integer; var r, g, b: integer);
begin
  r := c and 255;
  c := c shr 8;
  g := c and 255;
  c := c shr 8;
  b := c and 255;
end;

procedure TDSShape.Fill_HGradient;
var
  x, s: single;
  t, numsteps: integer;
  r, g, b, rs, gs, bs: single;
  sr, sg, sb: integer;
  er, eg, eb: integer;
begin
  numsteps := 255;
  s := width / numsteps;
  if s < 1 then begin
    s := 1;
    numsteps := width;
  end;
  getrgb(FFillColor, sr, sg, sb);
  getrgb(FFillColor2, er, eg, eb);
  if sr < er then
    rs := (er - sr) / numsteps
  else
    rs := 0 - ((sr - er) / numsteps);
  if sg < eg then
    gs := (eg - sg) / numsteps
  else
    gs := 0 - ((sg - eg) / numsteps);
  if sb < eb then
    bs := (eb - sb) / numsteps
  else
    bs := 0 - ((sb - eb) / numsteps);
  r := sr;
  g := sg;
  b := sb;
  with FDrawBitmap.canvas do begin
    x := 0;
    brush.style := bsSolid;
    pen.style := psClear;
    for t := 1 to numsteps do begin
      brush.color := rgb(round(r), round(g), round(b));
      fillrect(rect(round(x), 0, round(x + s + 2), height));
      r := r + rs;
      g := g + gs;
      b := b + bs;
      x := x + s;
    end;
  end;
end;

procedure TDSShape.Fill_VGradient;
var
  y, s: single;
  t, numsteps: integer;
  r, g, b, rs, gs, bs: single;
  sr, sg, sb: integer;
  er, eg, eb: integer;
begin
  numsteps := 255;
  s := height / numsteps;
  if s < 1 then begin
    s := 1;
    numsteps := height;
  end;
  getrgb(FFillColor, sr, sg, sb);
  getrgb(FFillColor2, er, eg, eb);
  if sr < er then
    rs := (er - sr) / numsteps
  else
    rs := 0 - ((sr - er) / numsteps);
  if sg < eg then
    gs := (eg - sg) / numsteps
  else
    gs := 0 - ((sg - eg) / numsteps);
  if sb < eb then
    bs := (eb - sb) / numsteps
  else
    bs := 0 - ((sb - eb) / numsteps);
  r := sr;
  g := sg;
  b := sb;
  with FDrawBitmap.canvas do begin
    y := 0;
    brush.style := bsSolid;
    pen.style := psClear;
    for t := 1 to numsteps do begin
      brush.color := rgb(round(r), round(g), round(b));
      fillrect(rect(0, round(y), width, round(y + s + 1)));
      r := r + rs;
      g := g + gs;
      b := b + bs;
      y := y + s;
    end;
  end;
end;

procedure TDSShape.CalcPaintRect;
var
  ox, oy: integer;
begin
  if FLineWidth = 1 then
    FPaintRect := Rect(0, 0, width, height)
  else begin
    ox := FLineWidth div 2;
    oy := FLineWidth div 2;
    if FLineWidth mod 2 > 0 then
      FPaintRect := Rect(ox, oy, width - ox, height - oy)
    else
      FPaintRect := Rect(ox, oy, width - ox + 1, height - oy + 1);
  end;
end;

procedure TDSShape.Paint;
var
  FillDraw: boolean;
begin
  if FStarted then begin
    CalcPaintRect;
    {with Canvas do begin
      pen.style := psClear;
      brush.color := clRed;
      brush.Style := bsSolid;
      fillrect(clientrect);
    end;}
    FillDraw := (FFillStyle <> fsSolid) and (FFillStyle <> fsNone);
    case FShapeType of
      stRectangle:
        if not FillDraw then
          Draw_Rectangle
        else
          FillDraw_Rectangle;
      stSquare:
        if not FillDraw then
          Draw_Square
        else
          FillDraw_Square;
      stEllipse:
        if not FillDraw then
          Draw_Ellipse
        else
          FillDraw_Ellipse;
      stCircle:
        if not FillDraw then
          Draw_Circle
        else
          FillDraw_Circle;
      stVLine:           Draw_VLine;
      stHLine:           Draw_HLine;
      stHorizontalLine1: Draw_HorizontalLine1;
      stHorizontalLine2: Draw_HorizontalLine2;
    end;
  end;
end;

procedure TDSShape.SetPenBrush;
begin
  with Canvas do begin
    if FLineWidth = 0 then
      pen.style := psClear
    else begin
      pen.color := FLineColor;
      pen.style := psSolid;
      pen.width := FLineWidth;
    end;
    brush.color := FFillColor;
    if FFillStyle = fsSolid then
      brush.Style := bsSolid
    else
      brush.Style := bsClear;
  end;
end;

procedure TDSShape.SetMaskPenBrush;
begin
  with FMaskBitmap.Canvas do begin
    if FLineWidth = 0 then
      pen.style := psClear
    else begin
      pen.color := clBlack;
      pen.style := psSolid;
      pen.width := FLineWidth;
    end;
    brush.color := clBlack;
    brush.Style := bsSolid;
  end;
  with FDrawBitmap.Canvas do begin
    if FLineWidth = 0 then
      pen.style := psClear
    else begin
      pen.color := FLineColor;
      pen.style := psSolid;
      pen.width := FLineWidth;
    end;
    brush.color := FFillColor;
    if FFillStyle = fsSolid then
      brush.Style := bsSolid
    else
      brush.Style := bsClear;
  end;
end;

procedure TDSShape.Draw_Rectangle;
begin
  with Canvas do begin
    SetPenBrush;
    rectangle(FPaintRect.Left, FPaintRect.Top, FPaintRect.Right, FPaintRect.Bottom);
  end;
end;

procedure TDSShape.Draw_Square;
var
  w, h, d: integer;
begin
  with Canvas do begin
    SetPenBrush;
    w := FPaintRect.Right - FPaintRect.Left;
    h := FPaintRect.Bottom - FPaintRect.Top;
    if w < h then begin
      d := (h - w) div 2;
      rectangle(FPaintRect.Left, FPaintRect.Top + d, FPaintRect.Right, FPaintRect.Top + d + w);
    end
    else begin
      d := (w - h) div 2;
      rectangle(FPaintRect.Left + d, FPaintRect.Top, FPaintRect.Left + d + h, FPaintRect.Bottom);
    end;
  end;
end;

procedure TDSShape.Draw_Ellipse;
begin
  with Canvas do begin
    SetPenBrush;
    ellipse(FPaintRect.Left, FPaintRect.Top, FPaintRect.Right, FPaintRect.Bottom);
  end;
end;

procedure TDSShape.Draw_Circle;
var
  w, h, d: integer;
begin
  with Canvas do begin
    SetPenBrush;
    w := FPaintRect.Right - FPaintRect.Left;
    h := FPaintRect.Bottom - FPaintRect.Top;
    if w < h then begin
      d := (h - w) div 2;
      ellipse(FPaintRect.Left, FPaintRect.Top + d, FPaintRect.Right, FPaintRect.Top + d + w);
    end
    else begin
      d := (w - h) div 2;
      ellipse(FPaintRect.Left + d, FPaintRect.Top, FPaintRect.Left + d + h, FPaintRect.Bottom);
    end;
  end;
end;

procedure TDSShape.Draw_VLine;
var
  x: integer;
begin
  x := (FPaintRect.Right - FPaintRect.Left) div 2;
  with Canvas do begin
    SetPenBrush;
    moveto(x, 0);
    lineto(x, height);
  end;
end;

procedure TDSShape.Draw_HLine;
var
  y: integer;
begin
  y := (FPaintRect.Bottom - FPaintRect.Top) div 2;
  with Canvas do begin
    SetPenBrush;
    moveto(0, y);
    lineto(width, y);
  end;
end;

procedure TDSShape.Draw_HorizontalLine1;
begin
  with Canvas do begin
    SetPenBrush;
    moveto(0, 0);
    lineto(width, height);
  end;
end;

procedure TDSShape.Draw_HorizontalLine2;
begin
  with Canvas do begin
    SetPenBrush;
    moveto(width, 0);
    lineto(0, height);
  end;
end;

procedure TDSShape.FillDraw_Rectangle;
begin
  CreateMaskBitmap;
  FillDrawBitmap;
  with FDrawBitmap.canvas do begin
    SetMaskPenBrush;
    rectangle(FPaintRect.Left, FPaintRect.Top, FPaintRect.Right, FPaintRect.Bottom);
  end;
  canvas.Draw(0, 0, FDrawBitmap);
  FDrawBitmap.Free;
end;

procedure TDSShape.FillDraw_Square;
var
  w, h, d: integer;
begin
  CreateMaskBitmap;
  FillDrawBitmap;
  with FMaskBitmap.canvas do begin
    SetMaskPenBrush;
    w := FPaintRect.Right - FPaintRect.Left;
    h := FPaintRect.Bottom - FPaintRect.Top;
    if w < h then begin
      d := (h - w) div 2;
      rectangle(FPaintRect.Left, FPaintRect.Top + d, FPaintRect.Right, FPaintRect.Top + d + w);
      MaskDrawBitmap;
      FDrawBitmap.canvas.rectangle(FPaintRect.Left, FPaintRect.Top + d, FPaintRect.Right, FPaintRect.Top + d + w)
    end
    else begin
      d := (w - h) div 2;
      rectangle(FPaintRect.Left + d, FPaintRect.Top, FPaintRect.Left + d + h, FPaintRect.Bottom);
      MaskDrawBitmap;
      FDrawBitmap.canvas.rectangle(FPaintRect.Left + d, FPaintRect.Top, FPaintRect.Left + d + h, FPaintRect.Bottom);
    end;
  end;
  DrawTransparent;
  FDrawBitmap.Free;
end;

procedure TDSShape.FillDraw_Ellipse;
var
  w, h, d: integer;
begin
  CreateMaskBitmap;
  FillDrawBitmap;
  with FMaskBitmap.canvas do begin
    SetMaskPenBrush;
    ellipse(FPaintRect.Left, FPaintRect.Top, FPaintRect.Right, FPaintRect.Bottom);
    MaskDrawBitmap;
    FDrawBitmap.canvas.ellipse(FPaintRect.Left, FPaintRect.Top, FPaintRect.Right, FPaintRect.Bottom);
  end;
  DrawTransparent;
  FDrawBitmap.Free;
end;

procedure TDSShape.FillDraw_Circle;
var
  w, h, d: integer;
begin
  CreateMaskBitmap;
  FillDrawBitmap;
  with FMaskBitmap.canvas do begin
    SetMaskPenBrush;
    w := FPaintRect.Right - FPaintRect.Left;
    h := FPaintRect.Bottom - FPaintRect.Top;
    if w < h then begin
      d := (h - w) div 2;
      ellipse(FPaintRect.Left, FPaintRect.Top + d, FPaintRect.Right, FPaintRect.Top + d + w);
      MaskDrawBitmap;
      FDrawBitmap.canvas.ellipse(FPaintRect.Left, FPaintRect.Top + d, FPaintRect.Right, FPaintRect.Top + d + w)
    end
    else begin
      d := (w - h) div 2;
      ellipse(FPaintRect.Left + d, FPaintRect.Top, FPaintRect.Left + d + h, FPaintRect.Bottom);
      MaskDrawBitmap;
      FDrawBitmap.canvas.ellipse(FPaintRect.Left + d, FPaintRect.Top, FPaintRect.Left + d + h, FPaintRect.Bottom);
    end;
  end;
  DrawTransparent;
  FDrawBitmap.Free;
end;

procedure Register;
begin
  RegisterComponents('Digis', [TDSShape]);
end;

end.
